@@ -40,24 +40,24 @@ Summary:
 
 fetch_genes() is an EXPLORATORY tool and makes a number of simplicifications:
 
-  - the DMR regions file saves one CpG probe name and location, even though clusters of probes may map to
-  that nearby area.
-  - it measures the distance from the start position of the one representative probe per region to any nearby
-  genes, using the `tol`erance parameter as the cutoff. Tolerance is the max number of base pairs of separation
-  between the probe sequence start and the gene sequence start for it to be considered as a match.
-  - The default `tol`erance is 250, but that is arbitrary. Increase it to expand the search area, or decrease it
-  to be more conservative. Remember that Illumina CpG probe sequences are 50 base pairs long, so 100 is nearly
-  overlapping. 300 or 500 would also be reasonable.
-  - "Adjacent" in the linear sequence may not necessarily mean that the CpG island is FUNCTIONALLY coupled to the
-  regulatory or coding region of the nearby protein. DNA superstructure can position regulatory elements near to
-  a coding region that are far upstream or downstream from the mapped position, and there is no easy way to identify
-  "adjacent" in this sense.
-  - Changing the `tol`erance, or the reference database will result major differences in the output, and thus
-  one's interpretation of the same data.
-  - Before interpreting these "associations" you should also consider filtering candidate genes by
-  specific cell types where they are expressed. You should know the tissue from which your samples originated.
-  And filter candidate genes to exclude those that are only expressed in your tissue during development,
-  if your samples are from adults, and vice versa.
+  * the DMR regions file saves one CpG probe name and location, even though clusters of probes may map to
+    that nearby area.
+  * it measures the distance from the start position of the one representative probe per region to any nearby
+    genes, using the `tol`erance parameter as the cutoff. Tolerance is the max number of base pairs of separation
+    between the probe sequence start and the gene sequence start for it to be considered as a match.
+  * The default `tol`erance is 250, but that is arbitrary. Increase it to expand the search area, or decrease it
+    to be more conservative. Remember that Illumina CpG probe sequences are 50 base pairs long, so 100 is nearly
+    overlapping. 300 or 500 would also be reasonable.
+  * "Adjacent" in the linear sequence may not necessarily mean that the CpG island is FUNCTIONALLY coupled to the
+    regulatory or coding region of the nearby protein. DNA superstructure can position regulatory elements near to
+    a coding region that are far upstream or downstream from the mapped position, and there is no easy way to identify
+    "adjacent" in this sense.
+  * Changing the `tol`erance, or the reference database will result major differences in the output, and thus
+    one's interpretation of the same data.
+  * Before interpreting these "associations" you should also consider filtering candidate genes by
+    specific cell types where they are expressed. You should know the tissue from which your samples originated.
+    And filter candidate genes to exclude those that are only expressed in your tissue during development,
+    if your samples are from adults, and vice versa.
 
 Arguments:
 
@@ -194,15 +194,34 @@ Arguments:
 
     # add column(s) for gene tissue expression
     if tissue != None:
-        # tissue == 'all'
-        tissues = fetch_genes(sql="select * from hgFixed.gtexTissueV8;")
+        tissue_cache = False
+        if no_sync == True: # avoid hitting UCSC server here and use the cached copy
+            tissue_names_file = Path(package_path, 'data', f"gtexTissueV8_names.pkl")
+            tissues_file = Path(package_path, 'data', f"gtexTissueV8.pkl")
+            tissue_names_available = tissue_names_file.exists()
+            tissues_available = tissues_file.exists()
+            if not tissues_available and tissue_names_available:
+                LOGGER.warning("No cache gtex Tissues data available; downloading from UCSC.")
+            else:
+                tissues = pd.read_pickle(tissue_names_file)
+                tissue_cache = True
+        else:
+            # tissue == 'all'
+            tissues = fetch_genes(sql="select * from hgFixed.gtexTissueV8;")
         sorted_tissues = [i['name'] for i in tissues]
         gene_names = [i.split(',') for i in list(regions['genes']) if i != '']
         N_regions_with_multiple_genes = len([i for i in gene_names if len(i) > 1])
         if N_regions_with_multiple_genes > 0:
             LOGGER.warning(f"{N_regions_with_multiple_genes} of the {len(gene_names)} regions have multiple genes matching in the same region, and output won't show tissue expression levels.")
         gene_names = tuple([item for sublist in gene_names for item in sublist])
-        gtex = fetch_genes(sql=f"select name, expScores from gtexGeneV8 WHERE name in {gene_names} and score > 0;")
+
+        # next, filter the full database table by gene_names with scores
+        if tissue_cache:
+            annot = pd.read_pickle(tissues_file) # pulled with "where score > 0"
+            # annot is a list of dicts; i['score'] is a stringified list of numbers, matching order of sorted tissues
+            gtex = [i for i in annot if i['name'] in gene_names]
+        else: # SLOWER; use live database by default
+            gtex = fetch_genes(sql=f"select name, expScores from gtexGeneV8 WHERE name in {gene_names} and score > 0;")
         if len(gtex) > 0:
             # convert to a lookup dict of gene name: list of tissue scores
             gtex = {item['name']: [float(i) for i in item['expScores'].decode().split(',') if i != ''] for item in gtex}
